"""Controller for Plugin logic

Used by plugins for setting and notification functionality.
"""

import sqlalchemy

from typing import Union
from flask import current_app
from werkzeug.exceptions import NotFound
from importlib.metadata import entry_points

from .. import logger
from ..database import db
from ..utils import Hook
from ..models import Plugin, PluginSetting, Notification


def get_setting(plugin_id: str, name: str, **kwargs):
    """Get plugin setting from database

    Args:
        plugin_id: ID of the plugin
        name: string identifying the setting
        default: Default value
    Returns:
        Value stored in database (native python)
    Raises:
        `KeyError` if no such setting exists in the database
    """
    try:
        setting = (
            PluginSetting.query.filter(PluginSetting.plugin == plugin_id).filter(PluginSetting.name == name).one()
        )
        return setting.value
    except sqlalchemy.orm.exc.NoResultFound:
        if "default" in kwargs:
            return kwargs["default"]
        else:
            raise KeyError


def set_setting(plugin_id: str, name: str, value):
    """Save setting in database

    Args:
        plugin_id: ID of the plugin
        name: String identifying the setting
        value: Value to be stored
    """
    setting = (
        PluginSetting.query.filter(PluginSetting.plugin == plugin_id)
        .filter(PluginSetting.name == name)
        .one_or_none()
    )
    if setting is not None:
        if value is None:
            db.session.delete(setting)
        else:
            setting.value = value
    else:
        db.session.add(PluginSetting(plugin=plugin_id, name=name, value=value))
    db.session.commit()


def notify(plugin_id: str, user, text: str, data=None):
    """Create a new notification for an user

    Args:
        plugin_id: ID of the plugin
        user: `flaschengeist.models.user.User` to notify
        text: Visibile notification text
        data: Optional data passed to the notificaton
    Returns:
        ID of the created `flaschengeist.models.notification.Notification`

    Hint: use the data for frontend actions.
    """
    if not user.deleted:
        n = Notification(text=text, data=data, plugin=plugin_id, user_=user)
        db.session.add(n)
        db.session.commit()
        return n.id
